


//class definition file...

struct CUSTOMVERTEX
{
    D3DXVECTOR3 p; // The semi-transformed position for the vertex
    D3DCOLOR    color;    // The color
    FLOAT       tu, tv;   // The texture coordinates
};

#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)

struct CUSTOM2DVERTEX
{
    float x,y,z; // The semi-transformed position for the vertex
	float rhw;
    D3DCOLOR    color;    // The color
    FLOAT       tu, tv;   // The texture coordinates
};

#define D3DFVF_CUSTOM2DVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1)

//#####################################################################################################################################################################
// CUSTOM_TRANSFORMED_VERTEX class - holds a transformed vertex...

struct TRANSFORMEDVERTEX
{
    float x,y,z; // The semi-transformed position for the vertex
	float rhw;
    D3DCOLOR    color;    // The color
    FLOAT       tu, tv;   // The texture coordinates
};

#define D3DFVF_TRANSFORMEDVERTEX (D3DFVF_XYZRHW |D3DFVF_DIFFUSE|D3DFVF_TEX1)


class TEXTURE_WRAPPER {
	public:

		int loaded_ok;
		char name[256];
		LPDIRECT3DTEXTURE8	texture;

		void set(void);

};




//VECTOR class - just what it looks like Anders ;-)

class VECTOR {
	public:

		float x,y,z;
		float nx,ny,nz;
		float nrx,nry,nrz;

		void Normalize(void);
		void MakeUnit(void);
		
		void set(float,float,float);
		float length(void);

};



//CAMERA class - a camera object for scene viewing position and angle - all matrices come from this virtual camera...

class CAMERA {
	public:

		void handle(void);
		void work_out_trig(void);

		float x,y,z,cx,cy,cz;
		float ax,ay,az;
		float oax,oay,oaz;

		float dax,day,daz;

		float COSax,COSay,COSaz;
		float SINax,SINay,SINaz;
//trigonometry values...
		float MINCOSax,MINCOSay,MINCOSaz;
		float MINSINax,MINSINay,MINSINaz;

		int windowx,windowy,windowsx,windowsy;

		float dx,dy,dz;

		float resx,resy, Tshift, Lshift;

		float zoom, fcp;


		D3DXMATRIX DMXtranslation,DMXrotation,DMXtotalTransformation,DMXidentity;

};





//DISPLAY class - contains all display/drawing structures and variables...

class DISPLAY {
	public:

		void	redraw(void);
		BOOL	SetMode(int width, int height, int bpp);

		void	initCamera(int newresx, int newresy, float newx, float newy, float newz, float newax, float neway, float newaz, float newzoom);

		VECTOR	light_source_direction;
		CAMERA	camera;

		HDC		hdc;

		float	avtime;

		CUSTOMVERTEX *t_vb;
		CUSTOM2DVERTEX *t_2Dvb;

		D3DXMATRIX matView;
		
		CD3DFont*	m_pFont;              // Font for drawing text
		CD3DFont*	HUDFont;              // Font for drawing text

};


//MOUSE class - contains state of mouse, including all buttons and position states...

class MOUSE {
	public:

		void handle(void);
		void un_click(void);
		void load_cursors(void);
		void find_3d_pos(void);
		void draw_cursor(void);

		unsigned short int Lb,Lclicked;
		unsigned short int Rb,Rclicked;
		unsigned short int Mb,Mclicked;
		unsigned short int Ldrag,Ldragged;
		unsigned short int Rdrag,Rdragged;

//standard mouse position...		
		float x,y, ox,oy;
//window-resolution mouse position...
		float mx,my,omx,omy;
//window-from-centre mouse position...
		float cmx,cmy,ocmx,ocmy;

		float Ldmx,Ldmy,Rdmx,Rdmy;

		float gridx,gridy,gridz;

		int mgridx,mgridy;


		int cursor;
		float cur_offx, cur_offy;


		TEXTURE_WRAPPER *cur_cursor;

		TEXTURE_WRAPPER arrow,select,move,attack,link,unlink,scavenge,deposit;
};


//KEYBOARD class - all key states and whether key has been pressed recently...

class KEYBOARD {
	public:

		void handle(void);

		unsigned short keydown[256];
		unsigned short keypressed[256];

};


//CONTROL class - encapsulates all other control specific classes...

class CONTROL {
	public:

		MOUSE		mouse;
		KEYBOARD	keyboard;

};


#include "geometry.h"



//###############################################################################################################################################################
//RESOURCES class - contains EVERY once instanced objects, animation, and texture - every resource the app needs...

class RESOURCES {
	public:

	struct			_finddata_t c_file[250];
	long			hFile;


	S_MESH			*first, *last;
};


//###############################################################################################################################################################

class DEVICES {
	public:
		
		LPDIRECT3D8             g_pD3D; // Used to create the D3DDevice
		LPDIRECT3DDEVICE8       g_pd3dDevice; // Our rendering device
		
		LPDIRECT3DVERTEXBUFFER8 g_pVB; // Buffer to hold vertices
		LPDIRECT3DVERTEXBUFFER8 g_2DpVB; // Buffer to hold vertices
		LPDIRECT3DVERTEXBUFFER8 g_psbVB; // Buffer to hold vertices

		char root_path[256];
		
		LPD3DXSPRITE Xsprite;

		char szAppName[256];
		
		HINSTANCE  hInstApp;
		BOOL       fAppActive;
		BOOL       fAppPaused;
		HWND       hwndApp;
		HACCEL     hAccelApp;
		HFONT      AppFont;

		SIZE	   ScreenSize;
};


//###############################################################################################################################################################

//GAME overclass - contains every variable and class for the app to run - needed only once...
class CMapEditorDlg;

class GAME {
	public:

		void initialize(void);
		void end(void);

		CMapEditorDlg	*mainDialog;
		
		DEVICES		devices;
		DISPLAY		display;
		CONTROL		control;
		RESOURCES	resources;

};






//main function definitions...


#define PI 3.14159265358979323846f

#define DEG2RAD(x) ((x)*PI / 180.0f)
#define RAD2DEG(x) ((x)/PI * 180.0f)

#define	random2(min,max) ((rand() % (int)(((max)+1) - (min))) + (min))

#define getkey(n) ((GetAsyncKeyState(n)!=0)==1)

VECTOR CrossProduct( VECTOR v1, VECTOR v2 );
float DotProduct( VECTOR v1, VECTOR v2 );
VECTOR MWCrossProduct( VECTOR *v1, VECTOR *v2 );
float DotProduct( VECTOR *v1, VECTOR *v2 );

void txtLine(float x1, float y1, float x2, float y2, float R, float G, float B);
void draw3DLine(float ax,float ay,float az,float bx,float by,float bz,int r,int g,int b,int a);

float f_abs(float);


draw_point_sprite(float x, float y, float z, float r, float g, float b, float a, float sizex, float sizey, TEXTURE_WRAPPER *texture);

