

//#################################################################################################################################################################

class S_VERTEX;
class S_POLY;
class V_INTERSECTION;

//#################################################################################################################################################################

enum S_PRIMITIVE_TYPE	{	CUBE,	SPHERE,		EMPTY	};

class S_MESH {

friend class S_POLY;
friend class S_VERTEX;
friend void DISPLAY::redraw(void);


public:

//constructors...
	S_MESH(char []);
	S_MESH(S_PRIMITIVE_TYPE);
	S_MESH(S_MESH *copyMesh);

	void copyVisibleFrom(S_MESH *copyFromMesh );

//destructor...
	~S_MESH();

//cleans up and optimises the mesh...
	S_MESH *optimise(void);
//removes unused vertices from the mesh...	
	void removeUnusedVerts(void);
//makes sure all polygons have unique vertices...
	void makeVerticesUnique(S_MESH *newMesh);

	void ajoinPolygons( void );

//draws the mesh...
	void draw(void);
	void light(void);
	
	void rotatex(double a);
	void rotatez(double a);

	void translate(float x, float y, float z);
	
	void scale(float x, float y, float z);

	void retrieveBounds(void);

	S_MESH *booleanIntersect(S_MESH *mesh);

//adds a vertex to the list...
	void operator += (S_VERTEX *v);
//removes a vertex from the list...
	void operator -= (S_VERTEX *v);

//adds a polygon to the list...
	void operator += (S_POLY *p);

//adds a polygon to the list...
	void addPolygon(S_POLY *p);
	void removePolygon(S_POLY *p);
	
//removes a polygon from the list (leaves vertexes)...
	void operator -= (S_POLY *p);


	S_VERTEX *findVertex( float x, float y, float z );

	S_POLY	*polygon( int index );

	float	bb_left,bb_right,bb_upper,bb_lower,bb_back,bb_front;

protected:

//link-list nodes...
	S_POLY		*pfirst, *plast;
	S_MESH		*mlast, *mnext;

	int			numVerts,numPolys;

};


//#################################################################################################################################################################

class S_VERTEX {

public:

	friend class S_MESH;
	friend class S_EDGE;
	friend class S_POLY;
	friend void DISPLAY::redraw(void);

//constructor...
	S_VERTEX(float x, float y, float z, float tu = 0.0, float tv = 0.0);
	S_VERTEX(void);
//destructor...
	~S_VERTEX();

	operator += (S_EDGE *e);

	int hasAnySimiliarVerts( S_POLY *p2 );


	void set( D3DXVECTOR3 *point, D3DCOLOR *color);
	void set( float x, float y, float z );
	void retrieve( D3DXVECTOR3 *point, D3DCOLOR *color);
	void retrieve( float *x, float *y, float *z);

	int isBehind(S_POLY *poly);
	int	isInside(S_MESH *mesh, double weighting = 0.0);
	int	isOutside(S_MESH *mesh, double weighting = 0.0);

	int isBehind( VECTOR *normal, VECTOR *planeDist );

	int isSame(S_VERTEX *v);
	int isSame(S_VERTEX *v, double error);


	unsigned short int testFlag;


protected:

	double x,y,z;
	double r,g,b;
	double tu,tv;


	S_EDGE	*nextEdge;
	S_EDGE *lastEdge;


//link-list nodes...
//	S_VERTEX	*vlast, *vnext;

};

//#################################################################################################################################################################

class S_EDGE {

public:

	S_EDGE( int isFirstEdge );

	operator		+= (S_VERTEX *v);

	int				visible;

	int				leaving;

	int				intersectWithPoly( S_VERTEX *lastVertex, S_POLY *target, S_VERTEX *generatedList, int *index );
	int				compareWithPolygon( S_POLY *poly );

	int				crossesEdge( S_EDGE *edge );

	int				isFirstEdge;

	int				intersected;

	V_INTERSECTION	*v_int;
	S_VERTEX		*nextVertex, *lastVertex;
	S_EDGE			*firstEdge;
};

//#################################################################################################################################################################
enum INT_TYPE { INT_NORMAL, INT_FLOATING, INT_FLOATING_TWIN, INT_PROCESSED, INT_PROCESSED2 };
//
//
//
class V_INTERSECTION
{
public:

	INT_TYPE	type;

	int			index;

	S_VERTEX	*intersectPoint;
	S_EDGE		*edge;

//if there is another int-point with the same co-ords as this one (floating twin) -> point to it here...	
	int			floatingTwin;

	int			enteringFace;

};


//#################################################################################################################################################################

enum S_MESHPART {  S_MP_UNDECIDED, S_MP_TOOL, S_MP_OBJECT  };

class S_POLY {

public:

	friend class S_MESH;
	friend class S_EDGE;
	friend class S_VERTEX;
	friend void DISPLAY::redraw(void);

//constructor...
	S_POLY(S_VERTEX *v1, S_VERTEX *v2, S_VERTEX *v3, S_VERTEX *v4);
	S_POLY(void);
	S_POLY( S_POLY *copy );
//destructor...
	~S_POLY();

	operator += (S_VERTEX *v);

//draw...
	void draw(int r = -1, int g = -1, int b = -1);


	int hasAnySimiliarVerts( S_POLY *p2 );


	void reset( void );



	void intersectWithObject( S_MESH *target, S_MESHPART	meshPart );
	
	void intersectWithPoly( S_POLY *target,	S_MESHPART	meshPart );


	int	 isPointInside( float x, float y, float z );


	S_POLY	*findPlanarPolygon( S_MESH *mesh, int isSameMesh, S_POLY *endPoly );

	VECTOR	findNormal( void );

//boolean functions...	
	void splitAgainst( S_MESH *original, S_MESH *toolMesh, S_MESH *newMesh, int flipNewFaces );
	S_VERTEX *performIntersection( S_VERTEX *edgev1, S_VERTEX *edgev2, int *inTriangle );
	int isBehind(S_POLY *poly);

	void		checkForTwins( int thisIndex );

//geom ops...	
	void flip(void);
	int invisible,intersected;
	int addedAsUnintersected;
	int addedAsIntersected;

	int intersectedPointsGenerated;

	int flipAfter;

	int part;
	
	S_MESHPART	meshPart;


protected:

	int numEdges;

	
	S_VERTEX	*firstVertex, *lastVertex;

	S_EDGE		*lastEdge;


	V_INTERSECTION	v_ints[100];
	int				num_ints;


	S_MESH *parentMesh;

//link-list nodes...
	S_POLY	*plast, *pnext;

};

//#################################################################################################################################################################
