

#include "stdafx.h"
#include "Modeller.h"
#include "MapEditorDlg.h"

#include "d3dWindowMain.h"


LONG CALLBACK AppWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
void RenderFrame();

//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D(void)
{
 	int num_tex;
	char tex_file_name[256];
   
	// Create the D3D object.
    if( NULL == ( game.devices.g_pD3D = Direct3DCreate8( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Get the current desktop display mode, so we can set up a back
    // buffer of the same format
    D3DDISPLAYMODE d3ddm;
    if( FAILED( game.devices.g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice. Since we are now
    // using more complex geometry, we will create a device with a zbuffer.
	D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.BackBufferWidth = 640;
	d3dpp.BackBufferHeight = 480;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

// Create the D3DDevice
    game.devices.g_pD3D->CreateDevice(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,game.devices.hwndApp,D3DCREATE_SOFTWARE_VERTEXPROCESSING,&d3dpp,&game.devices.g_pd3dDevice);
    
// Counter-clockwise Back-Face-Culling
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );

// Turn off D3D lighting
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

// Turn on the zbuffer
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_LESSEQUAL );


    game.devices.g_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,   TRUE );

    game.devices.g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA  );
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

// Enable alpha testing (skips pixels with less than a certain alpha.)
  //  game.devices.g_pd3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, FALSE );
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_ALPHAREF,        0x08 );
    game.devices.g_pd3dDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATER );
    
	game.devices.g_pd3dDevice->SetRenderState( D3DRS_CLIPPING, TRUE );
	game.devices.g_pd3dDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	game.devices.g_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);





	// Create the vertex buffer.
    game.devices.g_pd3dDevice->CreateVertexBuffer( 100000 * sizeof(CUSTOMVERTEX),D3DUSAGE_DYNAMIC, D3DFVF_CUSTOMVERTEX,D3DPOOL_DEFAULT, &game.devices.g_pVB );
    game.devices.g_pd3dDevice->CreateVertexBuffer( 1000 * sizeof(CUSTOM2DVERTEX),D3DUSAGE_DYNAMIC, D3DFVF_CUSTOM2DVERTEX,D3DPOOL_DEFAULT, &game.devices.g_2DpVB );
    
	game.devices.g_pd3dDevice->SetStreamSource( 0, game.devices.g_pVB, sizeof(CUSTOMVERTEX) );
    game.devices.g_pd3dDevice->SetVertexShader( D3DFVF_CUSTOMVERTEX );


	game.initialize();

    game.display.m_pFont        = new CD3DFont( "Arial", 6, 6, D3DFONT_BOLD);

    if (game.display.m_pFont->InitDeviceObjects( game.devices.g_pd3dDevice ) != S_OK) game.end();
	if (game.display.m_pFont->RestoreDeviceObjects() != S_OK) game.end();

    game.display.HUDFont        = new CD3DFont( "Arial", 6, 4, 0);

    if (game.display.HUDFont->InitDeviceObjects( game.devices.g_pd3dDevice ) != S_OK) game.end();
	if (game.display.HUDFont->RestoreDeviceObjects() != S_OK) game.end();

    return S_OK;
}



/**************************************************************************
AppInit

  Description:
  This is called when the application is first loaded. It initializes
  all variables, registers the window class, and creates the main app
  window.
**************************************************************************/

BOOL AppInit(HINSTANCE hInst,HINSTANCE hPrev,int sw,LPSTR szCmdLine)
{
	WNDCLASS cls;
	


	sprintf(game.devices.szAppName, "MetalWar BUILD v0.1");

	/* Save instance handle for DialogBoxes */
	game.devices.hInstApp = hInst;
	
	if (!hPrev)
	{
		//***  Register a class for the main application window
		cls.hCursor        = LoadCursor(0,IDC_ARROW);
		
		//*** Just for fun, we'll draw our own spinning cube icon.
		cls.hIcon          = LoadIcon(hInst, "AppIcon");
		cls.lpszMenuName   = "AppMenu";
		cls.lpszClassName  = game.devices.szAppName;
		cls.hbrBackground  = (HBRUSH)GetStockObject(BLACK_BRUSH);
		cls.hInstance      = hInst;
		cls.style          = CS_VREDRAW | CS_HREDRAW;
		cls.lpfnWndProc    = (WNDPROC)AppWndProc;
		cls.cbClsExtra     = 0;
		cls.cbWndExtra     = 0;
		
		if (!RegisterClass(&cls))
			return FALSE;
	}
	
	game.devices.hAccelApp = LoadAccelerators(hInst, "AppAccel");
	
	game.devices.hwndApp = CreateWindowEx(
		WS_EX_TOPMOST,
		game.devices.szAppName,           // Class name
		game.devices.szAppName,           // Caption
		WS_POPUP,
		0, 0, 	       // Position
		800,600,	       // Size
		0,                   // Parent window (no parent)
		0,                   // use class menu
		hInst,               // handle to window instance
		0                    // no params to pass on
		);
	ShowWindow(game.devices.hwndApp,sw);
	UpdateWindow(game.devices.hwndApp);
	
		
	InitD3D();

	//DDinit("Voodoo");
	
	return TRUE;
}

/**************************************************************************
WinMain

  Description:
  The main procedure for the App.  After initializing, it just goes
  into a message-processing loop until it gets a WM_QUIT message.
**************************************************************************/
/*
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
{
	MSG     msg;
	
	
	HRESULT hr = CoInitialize(NULL);
	
	if (hr!=S_OK) game.end();
	
	//*** Call initialization procedure
	if (!AppInit(hInst,hPrev,sw,szCmdLine))
		return FALSE;


	//*** Polling messages from event queue until quit
	for (;;)
	{
		if (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				break;
			
			if (!game.devices.hwndApp || !TranslateAccelerator(game.devices.hwndApp, game.devices.hAccelApp, &msg))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else
		{
			game.display.redraw();
		}
	}
	
	return msg.wParam;
}*/








void GAME::end(void){

	PostQuitMessage(1);

}


/**************************************************************************
AppWndProc

  Description:
  Main window proc. Standard Windows fare.
  **************************************************************************/
  
  LONG CALLBACK AppWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
  {
	  int fwKeys,KEY1,KEY2;
	  switch (msg)
	  {
	  case WM_CREATE:
		  break;
		  
	  case WM_ACTIVATEAPP:
		  break;
		  
	  case WM_SETCURSOR:
		  break;
		  
	  case WM_ENTERMENULOOP:
		  break;
		  
	  case WM_EXITMENULOOP:
		  break;
		  
	  case WM_COMMAND:
		  switch(LOWORD(wParam))
		  {
			  
		  case WM_DESTROY:
			  game.devices.hwndApp = NULL;
			  PostQuitMessage(0);
			  break;
			  
		  case WM_PAINT:
			  break;
		  }
		  
		  case WM_LBUTTONDOWN:
			  fwKeys = wParam;        
				  if (!game.control.mouse.Lb){
					  game.control.mouse.Ldmx = game.control.mouse.x;
					  game.control.mouse.Ldmy = game.control.mouse.y;
				  }
				  game.control.mouse.Lb=1;
			  break;
			  
		  case WM_RBUTTONDOWN:
			  fwKeys = wParam;        
				  if (!game.control.mouse.Rb){
					  game.control.mouse.Rdmx = game.control.mouse.x;
					  game.control.mouse.Rdmy = game.control.mouse.y;
				  }
				  game.control.mouse.Rb=1;
			  	game.control.mouse.omx = game.control.mouse.mx;
				game.control.mouse.omy = game.control.mouse.my;
				break;
			  
		  case WM_MBUTTONDOWN:
			  fwKeys = wParam;        
				  game.control.mouse.Mb=1;
			  break;
			  
		  case WM_LBUTTONUP:
			  if (game.control.mouse.Lb==1) {
				  game.control.mouse.Lclicked = 1;
			  }
			  game.control.mouse.Lb=0;
			  break;
			  
		  case WM_RBUTTONUP:
			  if (game.control.mouse.Rb==1) {
				  game.control.mouse.Rclicked = 1;
			  }
			  game.control.mouse.Rb=0;
			  break;
			  
		  case WM_MBUTTONUP:
			  if (game.control.mouse.Mb==1) {
				  game.control.mouse.Mclicked = 1;
			  }
			  game.control.mouse.Mb=0;
			  break;
			  
		  case WM_KEYDOWN:
			  
//			  KEY1 = (int) wParam;
			  
//			  if (KEY1==RK_CONTROL) KEY_CONTR=1;
//			  if (KEY1==RK_SHIFT) KEY_SHIFT=1;
			  
			  break;
			  
		  case WM_KEYUP:
			  
//			  KEY2= (int) wParam;
			  
//			  if (KEY2==RK_CONTROL) KEY_CONTR=0;
//			  if (KEY2==RK_SHIFT) KEY_SHIFT=0;
			  
			  break;
			  
			  
			  
			  
	  }
	  return DefWindowProc(hwnd,msg,wParam,lParam);
  }
