
#include "stdafx.h"
#include "Modeller.h"
#include "MetalWar.h"


float distRayPlane(VECTOR vRayOrigin, VECTOR vnRayVector, VECTOR vnPlaneNormal, float planeD);
bool VertexInTriangle(VECTOR vtxPoint, VECTOR vtx0, VECTOR vtx1, VECTOR vtx2);
void createNewGeometry( S_MESH *newMesh, S_VERTEX *ov1, S_VERTEX *ov2, S_VERTEX *ov3, S_VERTEX *nv1, S_VERTEX *nv2,	S_POLY **p1, S_POLY **p2, S_POLY **p3, int flipNewFaces  );


S_MESH *tmesha;

//#################################################################################################################################################################
// booleanIntersect() - interesects the two meshes, only changing the parent mesh...

S_MESH *S_MESH::booleanIntersect(S_MESH *mesh){

//create a new mesh to hold the intersected one...
	S_MESH *newMesh = new S_MESH(EMPTY);


	tmesha = mesh;

//loop all triangles of parent mesh...
	S_POLY *cpoly = this->pfirst;

	do{

		if (!cpoly) break;

		cpoly->splitAgainst( this, mesh, newMesh, 0 );

		cpoly = cpoly->pnext;

	}while(1);


//loop all triangles of tool mesh...
	cpoly = mesh->pfirst;

	do{

		if (!cpoly) break;

		cpoly->splitAgainst( mesh, this, newMesh, 1 );

		cpoly = cpoly->pnext;

	}while(1);


//loop all triangles of parent mesh...
	cpoly = newMesh->pfirst;

	do{

		if (!cpoly) break;

		cpoly->splitAgainst( newMesh, this, newMesh, 0 );

		cpoly = cpoly->pnext;

	}while(1);


	newMesh->scale(0.99,0.99,0.99);

//loop all triangles of newly-created mesh...
	cpoly = newMesh->pfirst;

	do{

		if (!cpoly) break;



		if ( !cpoly->vv1->isInside(this)) {
			cpoly->invisible = 1;
		} else if ( !cpoly->vv2->isInside(this)) {
			cpoly->invisible = 1;
		} else if ( !cpoly->vv3->isInside(this)) {
			cpoly->invisible = 1;
		}


		cpoly = cpoly->pnext;

	}while(1);

	newMesh->scale(1.0/0.99,1.0/0.99,1.0/0.99);

	newMesh->copyVisibleFrom(this);

	newMesh = newMesh->optimise();

//delete the old mesh....
	delete this;

	return newMesh;
}

//#####################################################################################################################################################################

int S_VERTEX::isInside(S_MESH *mesh){

//loop all triangles of mesh...
	S_POLY *cpoly = mesh->pfirst;

	do{

		if (!cpoly) break;


		VECTOR	Edge1, Edge2;
		VECTOR	t_Normal;
		VECTOR	t_Origin;
		t_Origin.set(0.0, 0.0, 0.0);
		VECTOR  t_PlaneDist1;
		t_PlaneDist1.set(cpoly->vv1->x, cpoly->vv1->y, cpoly->vv1->z);
									
		Edge1.set(cpoly->vv2->x-cpoly->vv1->x, cpoly->vv2->y-cpoly->vv1->y, cpoly->vv2->z-cpoly->vv1->z);
		Edge2.set(cpoly->vv3->x-cpoly->vv1->x, cpoly->vv3->y-cpoly->vv1->y, cpoly->vv3->z-cpoly->vv1->z);
									
		t_Normal = MWCrossProduct(&Edge1, &Edge2);
		t_Normal.MakeUnit();

		VECTOR t_Vertex;
		t_Vertex.set( this->x - t_PlaneDist1.x, this->y - t_PlaneDist1.y, this->z - t_PlaneDist1.z);
		
		t_Vertex.MakeUnit();

		float t = DotProduct(t_Normal, t_Vertex);



		if ( t < -0.0001 ) return 0;



		cpoly = cpoly->pnext;

	}while(1);

return 1;
}


//#####################################################################################################################################################################

void S_POLY::splitAgainst( S_MESH *original, S_MESH *toolMesh, S_MESH *newMesh, int flipNewFaces ){

//loop all triangles of parent mesh...
	S_POLY *cpoly = toolMesh->pfirst;

	S_POLY *p1,*p2,*p3;

	do{

		if (!cpoly) break;

		int Vnv1,Vnv2,Vnv3;

		S_VERTEX *nv1 = cpoly->performIntersection( this->vv1, this->vv2, &Vnv1 );
		S_VERTEX *nv2 = cpoly->performIntersection( this->vv2, this->vv3, &Vnv2 );
		S_VERTEX *nv3 = cpoly->performIntersection( this->vv3, this->vv1, &Vnv3 );
	
		p1 = p2 = p3 = NULL;

		if (Vnv1 || Vnv2 || Vnv3){
			if (nv2 && nv3){
				createNewGeometry( newMesh, this->vv1, this->vv2, this->vv3, nv2, nv3,		&p1, &p2, &p3,	flipNewFaces );

				if (this->vv3->isInside(tmesha)) p3->invisible = 1;
				else {
					p1->invisible = 1;
					p2->invisible = 1;
				}

				if (toolMesh != tmesha) cpoly->invisible = 1;

			} else if (nv1 && nv2){
				createNewGeometry( newMesh, this->vv3, this->vv1, this->vv2, nv1, nv2,		&p1, &p2, &p3,	flipNewFaces  );

				if (this->vv2->isInside(tmesha)) p3->invisible = 1;
				else {
					p1->invisible = 1;
					p2->invisible = 1;
				}
				if (toolMesh != tmesha) cpoly->invisible = 1;
			
			} else if (nv1 && nv3){
				createNewGeometry( newMesh, this->vv2, this->vv3, this->vv1, nv3, nv1,		&p1, &p2, &p3,	flipNewFaces  );
			
				if (this->vv1->isInside(tmesha)) p3->invisible = 1;
				else {
					p1->invisible = 1;
					p2->invisible = 1;
				}
				if (toolMesh != tmesha) cpoly->invisible = 1;
			}
		} 


		cpoly = cpoly->pnext;

	}while(1);


}



//#################################################################################################################################################################
// checkIntersection() - checks if this edge intersects this polygon...

S_VERTEX *S_POLY::performIntersection( S_VERTEX *edgev1, S_VERTEX *edgev2, int *inTriangle ){

	*inTriangle = 0;

	VECTOR	Edge1, Edge2;
	VECTOR	t_Normal;
	VECTOR	t_Origin;
	t_Origin.set(edgev1->x,edgev1->y,edgev1->z);
	VECTOR  t_PlaneDist1;
	t_PlaneDist1.set(vv1->x, vv1->y, vv1->z);
							
	Edge1.set(vv2->x-vv1->x, vv2->y-vv1->y, vv2->z-vv1->z);
	Edge2.set(vv3->x-vv1->x, vv3->y-vv1->y, vv3->z-vv1->z);
							
	t_Normal = MWCrossProduct(&Edge1, &Edge2);
	t_Normal.MakeUnit();
							
	float t_PlaneDist2 = MWDotProduct( &t_PlaneDist1, &t_Normal);

	VECTOR    t_Vector;
	t_Vector.set( edgev2->x - edgev1->x, edgev2->y - edgev1->y, edgev2->z - edgev1->z );
		
	
	float t = distRayPlane( t_Origin, t_Vector, t_Normal, t_PlaneDist2);
			
				

	if (t > 0.001 && t < 0.999){

		VECTOR IP;
		IP.set ( t_Origin.x + (t_Vector.x * t),t_Origin.y + (t_Vector.y * t),t_Origin.z + (t_Vector.z * t) );
								
// Is the intersection point within the triangle ?
		VECTOR t_va,t_vb,t_vc;

		t_va.set(vv1->x,vv1->y,vv1->z);
		t_vb.set(vv2->x,vv2->y,vv2->z);
		t_vc.set(vv3->x,vv3->y,vv3->z);
					
		if (VertexInTriangle(IP, t_va, t_vb, t_vc) == true){
			
			S_VERTEX *newVert = new S_VERTEX( t_Origin.x + (t_Vector.x * t), t_Origin.y + (t_Vector.y * t), t_Origin.z + (t_Vector.z * t) );

			*inTriangle = 1;

//add a new vertex at the intersection position...
			return newVert;
		} else {
			S_VERTEX *newVert = new S_VERTEX( t_Origin.x + (t_Vector.x * t), t_Origin.y + (t_Vector.y * t), t_Origin.z + (t_Vector.z * t) );

			*inTriangle = 0;

//add a new vertex at the intersection position...
			return newVert;

		}
			
	}

	return NULL;
}


//#####################################################################################################################################################################


void createNewGeometry( S_MESH *newMesh, S_VERTEX *ov1, S_VERTEX *ov2, S_VERTEX *ov3, S_VERTEX *nv1, S_VERTEX *nv2,		S_POLY **p1, S_POLY **p2, S_POLY **p3, int flipNewFaces  ){

	*p1 = *p2 = *p3 = NULL;

	S_VERTEX *n_vv1 = new S_VERTEX( *ov1 );
	S_VERTEX *n_vv2 = new S_VERTEX( *ov2 );
	S_VERTEX *n_vv3 = new S_VERTEX( *ov3 );

	S_VERTEX *n_nv2 = new S_VERTEX( *nv1 );
	S_VERTEX *n_nv3 = new S_VERTEX( *nv2 );

	(*newMesh) += n_vv1;
	(*newMesh) += n_vv2;
	(*newMesh) += n_vv3;

	(*newMesh) += n_nv2;
	(*newMesh) += n_nv3;

	S_POLY *newPolyA, *newPolyB, *newPolyC;

	if (!flipNewFaces){
		newPolyA = new S_POLY( n_vv1, n_nv2, n_nv3 );
		newPolyB = new S_POLY( n_vv1, n_vv2, n_nv2 );
		newPolyC = new S_POLY( n_nv3, n_nv2, n_vv3 );
	} else {
		newPolyA = new S_POLY( n_vv1, n_nv2, n_nv3 );
		newPolyA->flip();
		newPolyB = new S_POLY( n_vv1, n_vv2, n_nv2 );
		newPolyB->flip();
		newPolyC = new S_POLY( n_nv3, n_nv2, n_vv3 );
		newPolyC->flip();
	}

	(*newMesh) += newPolyA;
	(*newMesh) += newPolyB;
	(*newMesh) += newPolyC;

	*p1 = newPolyA;
	*p2 = newPolyB;
	*p3 = newPolyC;
}



//#####################################################################################################################################################################


//#####################################################################################################################################################################
// MWCrossProduct() - works out the cross-product of the vector...

VECTOR MWCrossProduct( VECTOR *v1, VECTOR *v2 )
{
	
	VECTOR t_v;
	
	float X = (v1->y * v2->z) -  (v1->z * v2->y);
	float Y = (v1->z * v2->x) -  (v1->x * v2->z);
	float Z = (v1->x * v2->y) -  (v1->y * v2->x);

	t_v.set(X,Y,Z);

	return t_v;
}




//#####################################################################################################################################################################
// VertexInTriangle() - works out if the vertex is inside the boundaries of the triangle...

bool VertexInTriangle(VECTOR vtxPoint, VECTOR vtx0, VECTOR vtx1, VECTOR vtx2)
{
	double dAngle;
	
	VECTOR vec0;
	vec0.set(vtxPoint.x - vtx0.x, vtxPoint.y - vtx0.y,vtxPoint.z - vtx0.z);
	VECTOR vec1;
	vec1.set(vtxPoint.x - vtx1.x, vtxPoint.y - vtx1.y,vtxPoint.z - vtx1.z);
	VECTOR vec2;
	vec2.set(vtxPoint.x - vtx2.x, vtxPoint.y - vtx2.y,vtxPoint.z - vtx2.z);
	
	vec0.MakeUnit();
	vec1.MakeUnit();
	vec2.MakeUnit();
	
	dAngle =
		acos( MWDotProduct(&vec0,&vec1) ) + 
		acos( MWDotProduct(&vec1,&vec2) ) + 
		acos( MWDotProduct(&vec2,&vec0) ) ;
	
	if( fabs( dAngle - 2 * PI ) < 0.01 )
		return true;
	else
		return false;
}



//#####################################################################################################################################################################
// GetLength() - returns the magnitude of the components...

float GetLength(float x,float y,float z){
	if (x<0) x *= -1.0f;
	if (y<0) y *= -1.0f;
	if (z<0) z *= -1.0f;

	return sqrt ( (x*x) + (y*y) + (z*z) );
}



//#####################################################################################################################################################################
// distRayPlane() - finds the distance from the ray to the plane...

float distRayPlane(VECTOR vRayOrigin, VECTOR vnRayVector, VECTOR vnPlaneNormal, float planeD)
{
    float cosAlpha;
    float deltaD;

    cosAlpha = MWDotProduct( 
        &vnRayVector ,  &vnPlaneNormal );

    // parallel to the plane (alpha=90)
    if (cosAlpha==0) return -1.0f;

    deltaD = planeD - 
        MWDotProduct(&vRayOrigin,&vnPlaneNormal);
    
    return (deltaD/cosAlpha);
}

//#####################################################################################################################################################################

